.bi-valign-default {
    vertical-align: -.125em;
}

.bi-valign-top {
    vertical-align: top;
}

.bi-valign-middle {
    vertical-align: middle;
}

.bi-valign-bottom {
    vertical-align: bottom;
}

.bi-valign-text-top {
    vertical-align: text-top;
}

.bi-valign-text-bottom {
    vertical-align: text-bottom;
}

/* Material Design Icons have some internal
   "padding" in the svg, so we need some
   alignment adjustments of md icons */
.mdi.bi-valign-default,
.mdi.bi-valign-bottom,
.mdi.bi-valign-text-bottom {
    transform:translateY(10%);
}

.mdi.bi-valign-top,
.mdi.bi-valign-text-top {
    transform:translateY(-10%);
}